import pytest


def test_run_task_failure(ecs, instance, workspace, run):
    def run_task(self=ecs, **kwargs):
        self.stubber.activate()
        self.stubber.add_response(
            method="run_task",
            service_response={
                "tasks": [],
                "failures": [
                    {"arn": "failing-arn-1", "reason": "boom", "detail": "detailed boom 1"},
                    {"arn": "failing-arn-2", "reason": "boom", "detail": "detailed boom 2"},
                ],
            },
            expected_params={**kwargs},
        )
        response = self.client.run_task(**kwargs)
        self.stubber.deactivate()
        return response

    instance.run_launcher.ecs.run_task = run_task

    with pytest.raises(Exception) as ex:
        instance.launch_run(run.run_id, workspace)

    assert ex.match("Task failing-arn-1 failed because boom: detailed boom 1")
    assert ex.match("Task failing-arn-2 failed because boom: detailed boom 2")



def test_default(ecs, instance, launch_run):
    initial_task_definitions = ecs.list_task_definitions()["taskDefinitionArns"]

    launch_run(instance)

    # A new task definition is created
    task_definitions = ecs.list_task_definitions()["taskDefinitionArns"]
    assert len(task_definitions) == len(initial_task_definitions) + 1
    task_definition_arn = list(set(task_definitions).difference(initial_task_definitions))[0]
    task_definition = ecs.describe_task_definition(taskDefinition=task_definition_arn)
    container_definitions = task_definition["taskDefinition"]["containerDefinitions"]

    assert len(container_definitions) == 1
    assert not container_definitions[0].get("dependsOn")


def test_include_sidecars_with_depends_on(ecs, instance_cm, launch_run, task_definition):
    with instance_cm({"include_sidecars": True}) as instance:
        initial_task_definitions = ecs.list_task_definitions()["taskDefinitionArns"]

        launch_run(instance)

        # A new task definition is created
        task_definitions = ecs.list_task_definitions()["taskDefinitionArns"]
        assert len(task_definitions) == len(initial_task_definitions) + 1
        task_definition_arn = list(set(task_definitions).difference(initial_task_definitions))[0]
        task_definition = ecs.describe_task_definition(taskDefinition=task_definition_arn)
        container_definitions = task_definition["taskDefinition"]["containerDefinitions"]

        assert len(container_definitions) == 2
        for container_definition in container_definitions:
            if container_definition.get("name") == "run":
                assert container_definition.get("dependsOn")
